local decals = {}
local decors = {}

function load_decals()
	local files = love.filesystem.getDirectoryItems("decals")
	for i,v in ipairs(files) do
		decals[i] = {}
		decals[i].img = love.graphics.newImage("decals/"..v)
		decals[i].name = v
	end
	
	--[[
	add_decor(-32 + 16,288 + 128,"rug.png")
	add_decor(192,864,"scraps.png")
	add_decor(-416,1248,"scraps.png")
	add_decor(96+16,736+16+8,"shelf.png")
	add_decor(-96 - 32,992 - 32,"rug2.png")
	]]--
end

function decor_reset()
	decors = {}
end

function add_decor(x,y,name)
	for i,v in ipairs(decals) do
		if(v.name == name) then
			local d = {}
			d.x = x
			d.y = y
			d.id = i
			d.name = name
			d.w = decals[i].img:getWidth()
			d.h = decals[i].img:getHeight()
			
			table.insert(decors, d)
			
			return
		end
	end
end

function num_decors()
	return #decors
end

function get_decor(i)
	if(decors[i]) then
		local d = decors[i]
		return d.x, d.y, d.name
	end
end

function remove_key()
	for i,v in ipairs(decors) do
		if(v.name == "key.png") then
			table.remove(decors, i)
			return
		end
	end
end

function remove_decor(x,y)
	for i,v in ipairs(decors) do 
		local w, h = decals[v.id].img:getWidth(), decals[v.id].img:getHeight()
		if(CheckCollision(x,y,1,1,v.x,v.y,w,h)) then
			table.remove(decors, i)
			return
		end
	end
end

function decor_update(dt)

end

function decor_draw()
	for i,v in ipairs(decors) do
		if(on_screen(v.x,v.y,v.w,v.h)) then
			local img = decals[v.id].img
			love.graphics.draw(img, v.x, v.y) 
		end
	end
end

local selectOffset = 0

local selectedDecalName = ""
local selectedDecalImg = nil

function get_selected_decal()
	return selectedDecalName
end

function preview_selected_decal(x,y)
	if(selectedDecalImg) then
		love.graphics.setColor(1,1,1,0.25)
		love.graphics.draw(selectedDecalImg, x, y)
		love.graphics.setColor(1,1,1)
	end
end

function decal_select_update(dt)

end

function decal_select_draw()
	local totalH = selectOffset
	for i,v in ipairs(decals) do
		local w, h = v.img:getWidth(), v.img:getHeight()
		
		love.graphics.draw(v.img, 0, totalH)
		
		totalH = totalH + h 
	end
end

function decal_select_mousepressed(x,y,button)
	if(button == 1) then
		local totalH = selectOffset
	
		for i,v in ipairs(decals) do
			local w, h = v.img:getWidth(), v.img:getHeight()
			
			if(CheckCollision(x,y,1,1,0,totalH,w,h)) then
				selectedDecalName = v.name
				selectedDecalImg = v.img
			end
			
			totalH = totalH + h 
		end	
	
		state = "edit"
	end
end

function love.wheelmoved(x, y)
	if(state == "decalSelect") then
		selectOffset = selectOffset + y * 20
	end
end