
local mx, my = 0,0
local lastMx, lastMy = 0, 0

local gridSize = 32

local x, y = 0, 0 -- spawn cursor world x, y
local lastX, lastY = 0, 0

local addX, addY = 0, 0 -- add new floor at coords
local addW, addH = 0, 0

local tool = "select" -- add, select, resize, fountain
local selected = 1

local toolType = "floor" -- floor, gate, switch, candles, fountains

local placed = false

local text = "default"

local editorState = "edit"

local waitFrame = false

function editor_update(dt)
	x, y = get_mousePos() -- mouse world pos
	x = math.floor((x + gridSize/2) / gridSize) * gridSize
	y = math.floor((y + gridSize/2) / gridSize) * gridSize
	mx, my = love.mouse.getPosition()  -- mouse screen pos
	if(love.mouse.isDown(2)) then
		move_cam(lastMx - mx, lastMy - my)
	end
	
	if(toolType == "candle" or toolType == "switch" or toolType == "fountain" or toolType == "decor") then -- single point items
		if(love.keyboard.isDown("lshift") and love.mouse.isDown(1)) then
			remove_tool(x,y)

		elseif(love.mouse.isDown(1) and placed == false) then
			placed = true
			add_tool(x, y)
		elseif(love.mouse.isDown(1) == false) then
			placed = false
		elseif(lastX ~= x or lastY ~= y) then
			placed = false
		end

	elseif(toolType == "floor" or toolType == "gate") then -- rectangle items with w & h

		if(tool == "add") then

			if(love.mouse.isDown(1)) then
				addW = x - addX
				addH = y - addY
			else
				tool = "select"
				if(addW and addH > 0) then

					
					add_tool(addX, addY, addW, addH)

					addX = 0
					addY = 0
					addW = 0
					addH = 0
				end
			end
		elseif(tool == "select") then
			if(love.mouse.isDown(1)) then
			
				-- resize
				if not(selected == 0) then
					local sx, sy = get_mousePos() -- screen mouse x, y
					local border = 20
					
					local fx, fy, fw, fh = 0,0,0,0
					if(toolType == "floor") then 
						fx, fy, fw, fh = get_floor(selected)
					elseif(toolType == "gate") then
						fx, fy, fw, fh = get_gate(selected)
					end
					
					if(CheckCollision(sx,sy,1,1,fx - border,fy - border,fw + border * 2,fh + border * 2) and CheckCollision(sx,sy,1,1,fx,fy,fw,fh) == false) then -- mouse in border
						tool = "resize"
					end
				end
			
				if not(tool == "resize") then
					local sx, sy = get_mousePos() -- screen mouse x, y
					local prevSelected = selected
					
					if(toolType == "floor") then
						selected = get_floor_atPos(sx, sy)
					elseif(toolType == "gate") then
						selected = get_gate_atPos(sx, sy)
					end
					
					if(selected > 0 and selected == prevSelected) then
						tool = "move"
					
					elseif(selected == 0) then
						tool = "add"
					end
				end
			end
		elseif(tool == "resize") then
			if not(love.mouse.isDown(1)) then
				tool = "select"
			else
				local fx, fy, fw, fh = 0,0,0,0
				if(toolType == "floor") then
					fx, fy, fw, fh = get_floor(selected)
				elseif(toolType == "gate") then
					fx, fy, fw, fh = get_gate(selected)
				end
				
				local dx, dy =  x - lastX, y - lastY
				if(x < fx + fw/2) then
					if(toolType == "floor") then
						move_floor(selected, dx, 0)
						resize_floor(selected, -dx*2, 0)
					elseif(toolType == "gate") then
						move_gate(selected, dx, 0)
						resize_gate(selected, -dx*2, 0)				
					end
				end
				if(y < fy + fh/2) then
					if(toolType == "floor") then
						move_floor(selected, 0, dy)
						resize_floor(selected, 0, -dy*2)
					elseif(toolType == "gate") then
						move_gate(selected, 0, dy)
						resize_gate(selected, 0, -dy*2)				
					end
				end
				if(toolType == "floor") then
					resize_floor(selected, dx, dy)
				elseif(toolType == "gate") then
					resize_gate(selected, dx, dy)
				end
			end
		elseif(tool == "move") then
			if not(love.mouse.isDown(1)) then
				tool = "select"
			else
				if(toolType == "floor") then
					move_floor(selected, x - lastX, y - lastY)
				elseif(toolType == "gate") then
					move_gate(selected, x - lastX, y - lastY)
				end
			end
		end
	end
	
	lastMx, lastMy = mx, my
	lastX, lastY = x, y
	
	waitFrame = false
end

function add_tool(x, y, w, h)
	if(waitFrame == false) then
	
		if(toolType == "candle") then
			--local x, y = get_mousePos() 
			add_light(x,y, math.random(20,40)/10,{math.random(100,255)/255,math.random(100,255)/255, math.random(100,255)/255})
		elseif(toolType == "switch") then
			add_switch(x,y,text)
		elseif(toolType == "fountain") then
			add_fountain(x, y)
		elseif(toolType == "floor") then
			add_floor(x,y,w,h)
		elseif(toolType == "gate") then
			add_gate(x,y,w,h,text)
		elseif(toolType == "decor") then
			add_decor(x,y,get_selected_decal())
		end
	
	end
end

function remove_tool(x, y)
	if(toolType == "candle") then
		local x, y = get_mousePos() 
		remove_light(x,y)
	elseif(toolType == "switch") then
		local x, y = get_mousePos() 
		remove_switch(x,y)
	elseif(toolType == "fountain") then
		local x, y = get_mousePos() 
		remove_fountain(x,y)
	elseif(toolType == "decor") then
		local x, y = get_mousePos()
		remove_decor(x,y)
	end
end


function resize_tool()

end

function select_tool()
	if(toolType == "candle") then
	
	elseif(toolType == "switch") then
	
	end
end

function preview_tool()
	if(toolType == "decor") then
		preview_selected_decal(x,y)	
	end
end

function editor_mousepressed(sx,sy,button)
	if(button == 1) then
		addX, addY = x, y
		addW, addH = 0, 0
	end
end

function editor_keypressed(key)

	if(editorState == "edit") then
		if(key == "f1") then
			world_reset()	
		elseif(key == "f2") then
			save_lvl()
		elseif(key == "f3") then
			load_lvl()
		elseif(key == "o") then
			set_start(x, y)
		elseif(key == "p") then
			set_exit(x,y)
		elseif(key == "x") then
			if(toolType == "floor") then
				remove_floor(selected)
			elseif(toolType == "gate") then
				remove_gate(selected)
			end
		elseif(key == "1") then
			toolType = "floor"
		elseif(key == "2") then
			toolType = "gate"
		elseif(key == "3") then
			toolType = "candle"
		elseif(key == "4") then
			toolType = "switch"
		elseif(key == "5") then
			toolType = "fountain"
		elseif(key == "6") then
			toolType = "decor"
		elseif(key == "return") then
			editorState = "textEntry"
		elseif(toolType == "decor" and key == "d") then
			state = "decalSelect"
			waitFrame = true
		elseif(key == "s") then
			set_charPos(1, x, y)
		
		end
	elseif(editorState == "textEntry") then
		if(key == "backspace") then
			text = ""
		elseif(key == "return") then
			editorState = "edit"
		else
			text = text..key
		end
	end
end

function draw_grid()
	local cx, cy = get_cam()
	local w, h = love.graphics.getWidth(), love.graphics.getHeight()
	love.graphics.setColor(0, 0.5, 0, 0.5)
	for x = 0, w/gridSize + 10 do
		local xPos = math.floor((cx-w/2) / gridSize) * gridSize + x * gridSize
		local yPos = math.floor((cy-h/2) / gridSize) * gridSize
		love.graphics.line(xPos, yPos, xPos, yPos + h)
	end

	for y = 0, h/gridSize + 1 do
		--local yPos = (-cy % gridSize) + y * gridSize
		local xPos = math.floor((cx - w/2) / gridSize) * gridSize
		local yPos = math.floor((cy - h/2) / gridSize) * gridSize + y * gridSize
		love.graphics.line(xPos, yPos, xPos + w, yPos)
	end


	love.graphics.setColor(1,1,1)
end

function editor_draw()

	draw_grid()

	love.graphics.print("edit")
	love.graphics.setColor(0.5, 0, 0, 0.5)
	love.graphics.circle("fill",x,y,3)
	love.graphics.print(x..", "..y,x + 6,y)

	love.graphics.rectangle("fill", addX, addY, addW, addH)

	if(tool == "select") then
		if(selected ~= 0) then
			local x,y,w,h = get_floor(selected)
			love.graphics.setColor(0,1,0,0.5)
			love.graphics.rectangle("fill", x, y, w, h)
			love.graphics.setColor(1,1,1)
		end
	end
	
	preview_tool()

	love.graphics.setColor(1,1,1)
end

function editor_ui()
	love.graphics.print(tool..", "..toolType..", "..text,100,0)
end

function editor_start()
	state = "edit"
	load_all_floors()
end

local editor_pause = 0

function get_editor_pause()
	return editor_pause
end

function editor_pause_update(dt)
	editor_pause = math.max(0, editor_pause - dt)
end


function editor_stop()
	state = "play" 
	set_camPos(get_playerPos())
	trim_floors()
	editor_pause = 1
end

function save_lvl()
	local save = ""
	for i = 1, num_floors() do
		local x,y,w,h = get_floor(i)
		save = save.."add_floor("..x..","..y..","..w..","..h..");"
	end
	
	for i = 1, num_lights() do
		local x,y,w,h = get_light(i)
		save = save.."add_light("..x..","..y..");"	
	end
	
	for i = 1, num_gates() do
		local x,y,w,h,id = get_gate(i)
		save = save.."add_gate("..x..","..y..","..w..","..h..",'"..id.."');"
	end
	
	for i = 1, num_switches() do
		local x,y,id = get_switch(i)
		save = save.."add_switch("..x..","..y..",'"..id.."');"
	end	
	
	for i = 1, num_fountains() do
		local x,y,id = get_fountain(i)
		save = save.."add_fountain("..x..","..y..");"
	end	

	for i = 1, num_decors() do
		local x,y,name = get_decor(i)
		save = save.."add_decor("..x..","..y..",'"..name.."');"
	end	
	
	local sx, sy = get_start()
	save = save.."set_start("..sx..","..sy..");"
	sx, sy = get_exit()
	save = save.."set_exit("..sx..","..sy..");"
	love.filesystem.write("lvl.txt", save)
end

function load_lvl()
	decor_reset()
	world_reset()
	chars_reset()

	local load = love.filesystem.load("lvl.txt")
	if(load) then
		load()
	else
		add_floor(-400,-400,800,800)
	end

	trim_floors()
	player_setup()

	fountain_setup()
end