local glowImg = love.graphics.newImage("glow.png")
local smokeImg = love.graphics.newImage("smoke.png")
local smokeImg2 = love.graphics.newImage("smoke2.png")
glowImg:setFilter("linear","linear")
--smokeImg:setFilter("linear","linear")

local lights = {}
local orbs = {}
local sparks = {}

function remove_light(x,y)
	for i,v in ipairs(lights) do
		if(dist(x,y,v.x,v.y) <= 17) then
			table.remove(lights, i)
			break
		end
	end
end

function num_lights()
	return #lights
end

function get_light(i)
	if(lights[i]) then
		return lights[i].x, lights[i].y
	end
	return 0, 0
end

local lightDelay = 1 -- 0.5

function reset_lights(fullReset)
	--lights = {}
	for i,v in ipairs(lights) do
		v.r = math.random(20,40)/10
		v.spent = false
		v.delay = lightDelay
	end
	
	if(fullReset) then
		orbs = {}
		sparks = {}
	end
end

function add_spark(x,y,col,amount,angle,force,t,startXvel, startYvel)
	for i=1,amount do
		local s = {}
		s.spin = 0
		s.t = t
		s.x = x
		s.y = y
		s.z = 0
		s.size = math.random(5,10)/10
		s.lifetime = math.random(0,15)/10
		s.timer = s.lifetime
		s.col = {col[1], col[2], col[3]}
		local minAngle = angle - 0.3
		local maxAngle = angle + 0.3
		if(t == "smoke" or t == "blood" or t == "dust") then
			minAngle = 0
			maxAngle = 2 * math.pi
		end
		local f = math.random(force/2,force)
		local a = math.random(minAngle*100,maxAngle*100)/100
		s.a = a
		s.xvel = math.cos(a) * f --math.random(-40,40)
		s.yvel = math.sin(a) * f --math.random(-40,40)
		s.zvel = f * 1.4
		
		if(t == "smoke" or t == "blood" or t == "dust") then
			s.a = math.random(minAngle*100,maxAngle*100)/100
			s.size = s.size * 4--4
			s.lifetime = math.random(10,30)/10
			s.timer = s.lifetime - 0.25
			s.z = 40
			s.y = s.y + 40
			s.x = s.x + s.xvel * 0.1 + math.random(-5,5)
			s.y = s.y + s.yvel * 0.1 + math.random(-5,5)
			if(startXvel and startYvel) then
				s.xvel = s.xvel + startXvel
				s.yvel = s.yvel + startYvel
			end
		end
		
		if(t == "glow") then
			s.size = math.random(5,15)/10
		end
		
		table.insert(sparks, s)
	end
end

function add_orb(x,y,col,amount)
	for i=1,amount do
		local o = {}
		o.x = x
		o.y = y
		o.z = 0
		o.col = {col[1], col[2], col[3]}
		o.timer = 0
		o.xvel = math.random(-20,20)
		o.yvel = math.random(-20,20)
		o.zvel = math.random(95,120)
		o.bounces = 20
		table.insert(orbs, o)
	end
end



function add_light(x, y, r, col)
	local l = {}
	l.x = x
	l.y = y
	l.col = {math.random(100,255)/255,math.random(100,255)/255, math.random(100,255)/255}
	l.r =  math.random(20,40)/10
	l.delay = lightDelay
	if(r) then
		l.r = r
	end
	if(col) then
		l.col = {col[1], col[2], col[3]}
	end	
		l.spent = false
	table.insert(lights, l)
end

local flicker = 0

local drainRange = 80
local drainSpeed = 0.5
local fillAmount = 4
local minLight = 0.75

local orbGravity = 100
local orbPull = 6500
local orbMinBounce = 40
local orbPullAfter = 3

local sparkGravity = 100
local smokeRise = 5

function lights_update(dt)

	flicker = flicker + dt
	flicker = flicker % 1
	
	local px, py = get_playerPos()
	
	for i,v in ipairs(sparks) do
		local dt = dt
		if(v.t == "blood" and get_slowmo() > 0 ) then
			dt = dt * 5
		end
		v.spin = v.spin + dt/2
		v.timer = v.timer - dt
		
		if(v.t == "smoke" or v.t == "blood") then
			--v.zvel = v.zvel + 5 * dt
			v.zvel = 0
			v.z = v.z + smokeRise * dt
		else
			v.zvel = v.zvel - sparkGravity * dt
		end
		
		if(v.z + v.zvel * dt < 0) then
			v.zvel = v.zvel * 0.75
		end
		
		if(v.t == "blood" and get_slowmo() > 0 ) then
			v.x = v.x + v.xvel * dt * 0.1
			v.y = v.y + v.yvel * dt * 0.1
			v.z = v.z + v.zvel * dt * 0.1
		else
			v.x = v.x + v.xvel * dt
			v.y = v.y + v.yvel * dt
			v.z = v.z + v.zvel * dt
		end
		
		if(v.timer < 0) then
			table.remove(sparks, i)
		end
	end
	
	for i,v in ipairs(lights) do
		--[[
		if(dist(v.x, v.y, px, py) < drainRange and v.r > minLight) then
			v.r = v.r - drainSpeed * dt
			player_addLantern(fillAmount * dt)
		end
		]]
		if(v.spent and v.delay > 0) then
			v.delay = math.max(0, v.delay - dt)
			if(v.delay == 0) then
				add_orb(v.x, v.y, v.col, v.r * 2)
				add_spark(v.x,v.y , v.col, v.r * 7 * 2, 0, 40, "smoke")			
			end
		end
		if(v.spent == false and dist(v.x, v.y, px, py) < drainRange) then
			v.spent = true


			play_sfx("blowOut",v.x, v.y)
		elseif(v.delay == 0 and v.spent and v.r > minLight) then
			v.r = math.max(minLight, v.r + (minLight - v.r) * dt)
		end		
	end
	
	local px, py = get_playerPos()
	py = py - 30
	for i,v in ipairs(orbs) do
		v.timer = v.timer + dt
		if(v.timer > orbPullAfter) then
			local d = dist(v.x, v.y, px, py)
			local a = math.atan2(py - v.y, px - v.x)
			local dx = math.cos(a) * orbPull * 1/d * math.min(2, (v.timer-orbPullAfter + 1))
			local dy = math.sin(a) * orbPull * 1/d * math.min(2, (v.timer-orbPullAfter + 1))
			
			v.xvel = v.xvel + dx * dt
			v.yvel = v.yvel + dy * dt
			
			v.xvel = v.xvel - v.xvel * 0.5 * dt
			v.yvel = v.yvel - v.yvel * 0.5 * dt
			if(d < 20 and v.z < 50) then
				player_addLantern(fillAmount)
				add_spark(v.x, v.y , v.col, fillAmount*2, a + 3.14,50,"glow")
				play_sfx("ting",v.x, v.y)
				table.remove(orbs, i)
			end
		end
		
		v.zvel = v.zvel - orbGravity * dt
		if(v.z + v.zvel * dt < 0) then
			v.z = 0
			if(v.timer < orbPullAfter) then
				v.zvel = math.max(math.abs(v.zvel) * 0.75, orbMinBounce) -- * 0.5
				v.xvel = v.xvel * 0.75
				v.yvel = v.yvel * 0.75
				play_sfx("ting2",v.x, v.y)
			else
				v.zvel = 0
			end
		end
		
		local prevXvel, prevYvel = v.xvel, v.yvel
		v.x, v.y, v.xvel, v.yvel = world_bounceCol2(v.x, v.y, 2, 2, v.xvel, v.yvel, dt)
		if(prevXvel ~= v.xvel or v.yvel ~= prevYvel) then
			v.bounces = v.bounces - 1

			play_sfx("ting2",v.x, v.y)
			add_spark(v.x, v.y , v.col, fillAmount*2,math.atan2(v.yvel, v.xvel),50,"glow")
		end
		
		v.z = v.z + v.zvel * dt
		v.x = v.x + v.xvel * dt
		v.y = v.y + v.yvel * dt
		
		if(v.bounces <= 0) then
			table.remove(orbs, i)
		end		
	end
end

local candleImg = love.graphics.newImage("candle0.png")
local candleImg2 = love.graphics.newImage("candle1.png")

function lights_draw()
	world_createMesh()

	local px, py = get_playerPos()

	love.graphics.setColor(1,190/255,90/255)
	
	love.graphics.setColor(1,0.75,1)
	
	--love.graphics.setColor(1,80/255,90/255)
	light_draw(px, py, 1.5 + 6.5 * (math.log(player_getLantern() + 0.1) + 2)/2) -- player lantern light -- size 5.5

	--love.graphics.setColor(1,1,1)
	--light_draw(px, py, 2)

	for i = 1, num_fountains() do
		local x,y, fade = get_fountain(i)
		if(on_screen(x-300,y-300,600,600)) then
			love.graphics.setColor(182/255,1,235/255)
			light_draw(x, y - 30, fade * 5)
		end

	end

	for i=1,num_bloods() do
		local x,y,lifetime,collectible = get_blood(i)
		if(collectible) then
			love.graphics.setColor({0.5,0,0})
			light_draw(x, y, lifetime * 1, true)
		end
	end

	for i,v in ipairs(lights) do
		if(on_screen(v.x-v.r*50,v.y-v.r*50,v.r*100,v.r*100)) then
			love.graphics.setColor(v.col)
			light_draw(v.x, v.y, v.r)
		end
	end

	cam_set()
	for i,v in ipairs(orbs) do
		love.graphics.setColor(v.col)
		local r = 2
		love.graphics.draw(glowImg, v.x, v.y - v.z,0,r/2,r/2,glowImg:getWidth()/2, glowImg:getHeight()/2)
		r = 1
		love.graphics.draw(glowImg, v.x, v.y - v.z,0,r/2,r/2,glowImg:getWidth()/2, glowImg:getHeight()/2)
		love.graphics.circle("fill", v.x, v.y - v.z, 4)
	end
	cam_unset()
	love.graphics.setColor(1,1,1)
end

function lights_draw2()
	local px, py = get_playerPos()
	py = py - 20
	local r = 1.5 + 4.5 * player_getLantern()
	love.graphics.setColor(1,0.75,1, 0.25)
	love.graphics.draw(glowImg, px, py,0,r ,r,glowImg:getWidth()/2, glowImg:getHeight()/2)	
end

function lights_drawCandles()
	for i,v in ipairs(lights) do
		love.graphics.setColor(v.col[1]*1.5,v.col[2]*1.5,v.col[3]*1.5, 0.5)
		love.graphics.draw(glowImg, v.x, v.y,0,v.r/3 ,v.r/3,glowImg:getWidth()/2, glowImg:getHeight()/2)	
	
		love.graphics.setColor(v.col[1],v.col[2],v.col[3])
		if(math.floor(flicker*10) < 5) then
			love.graphics.draw(candleImg, v.x, v.y + 15,0, 1, 1, candleImg:getWidth()/2, candleImg:getHeight()/2)
		else
			love.graphics.draw(candleImg2, v.x, v.y + 15,0, 1, 1, candleImg2:getWidth()/2, candleImg2:getHeight()/2)
		end

	end
		
	
		
	love.graphics.setColor(1,1,1)	
end

function lights_drawOrbs()
	for i,v in ipairs(sparks) do
		if(v.t == "glow") then love.graphics.setBlendMode("add") else love.graphics.setBlendMode("alpha") end
		local img = glowImg
		if(v.t == "smoke") then img = smokeImg2 end
		if(v.t == "dust" or v.t == "blood") then img = smokeImg end
		love.graphics.setColor(v.col[1],v.col[2],v.col[3],v.timer/v.lifetime)
		if(v.t == "blood") then
			love.graphics.setColor(v.col[1],v.col[2],v.col[3],v.timer/v.lifetime * 2)
		end
		local r = (1 - v.timer/v.lifetime) * v.size * 2
		love.graphics.draw(img, v.x, v.y - v.z,v.a + v.spin,r/2,r/2,img:getWidth()/2, img:getHeight()/2)
		if(v.t == "glow") then
			r = r/2
			love.graphics.draw(img, v.x, v.y - v.z,v.a,r/2,r/2,img:getWidth()/2, img:getHeight()/2)
		end
		
		love.graphics.setColor(v.col[1]*3,v.col[2]*3,v.col[3]*3,v.timer/v.lifetime)
		--love.graphics.circle("fill", v.x, v.y - v.z, (1 - v.timer/v.lifetime) * v.size * 2)
		--love.graphics.circle("line", v.x, v.y - v.z, (1 - v.timer/v.lifetime) * v.size * 2)
	end	
	love.graphics.setBlendMode("alpha")
	for i,v in ipairs(orbs) do
		love.graphics.setColor(v.col[1], v.col[2], v.col[3], 0.5 + 0.25 * math.cos((flicker+i/10)*2*math.pi))
		local r = 1
		love.graphics.draw(glowImg, v.x, v.y - v.z,0,r/2,r/2,glowImg:getWidth()/2, glowImg:getHeight()/2)
		local r = 0.5
		love.graphics.draw(glowImg, v.x, v.y - v.z,0,r/2,r/2,glowImg:getWidth()/2, glowImg:getHeight()/2)
		love.graphics.circle("line", v.x, v.y - v.z, 4)
		love.graphics.circle("fill", v.x, v.y - v.z, 2)
	end	
	love.graphics.setColor(1,1,1)	
end

function light_draw(x, y, r, noFlicker)
	local function stencil()
		cam_set()

		world_shadows(x,y,r)

		cam_unset()
	end

	--love.graphics.stencil(stencil, "replace", 1)
	--love.graphics.setStencilTest("equal", 0)

	cam_set()

	--love.graphics.setColor(1,190/255,90/255)

	local flickerAmount = math.cos( flicker*2*math.pi) * 0.2
	if(noFlicker) then 
		love.graphics.draw(glowImg, x, y,0,r,r,glowImg:getWidth()/2, glowImg:getHeight()/2)
	else
		love.graphics.draw(glowImg, x, y,0,r + flickerAmount,r +flickerAmount,glowImg:getWidth()/2, glowImg:getHeight()/2)
		love.graphics.draw(glowImg, x, y,0,r/2 + flickerAmount,r/2 + flickerAmount,glowImg:getWidth()/2, glowImg:getHeight()/2)
	end
	--love.graphics.setColor(1,1,1,0.5)
	
	
	
	
	
	--love.graphics.setColor(1,1,1)
	--love.graphics.draw(glowImg, x, y,0,r,r,glowImg:getWidth()/2, glowImg:getHeight()/2)
	cam_unset()
end