love.graphics.setDefaultFilter("nearest", "nearest")

require "char"
require "player"
require "enemies"
require "drawables"
require "world"
require "hitbox"
require "spawner"
require "editor"
require "blood"
require "models"
require "physObj"
require "cam"
require "lights"
require "sfx"
require "fog"
require "tint"
require "decor"


function love.load()
	models_load()
	load_decals()

	love.window.setVSync(0)
	
	load_lvl()

	--add_light(200,200, 2)
	--light_floors()
	
	--add_gate(100,100,32,32*5,"1")
	--add_switch(-100,100,"1")
	
	
end

local cam = {x=0, y=0}

state = "play"
local hitstop = 0
local slowmo = 0

function get_slowmo()
	return slowmo
end

function add_hitstop(x,y)
	hitstop = x
	slowmo = slowmo + y
end
local frameCounter = 0

local endScreenFade = 0

function love.update(dt)


	editor_pause_update(dt)

	if(state == "play" and dt < 1) then
		if(hitstop > 0) then
			hitstop = math.max(0,hitstop - dt)
			dt = 0
		elseif(slowmo > 0 or love.keyboard.isDown("lshift") or player_isAlive() == false) then
			slowmo = math.max(0,slowmo - dt)
			dt = dt / 10
		elseif(love.keyboard.isDown("lctrl"))then
			dt = dt * 10
		end	
	
		if(love.keyboard.isDown("p")) then
			add_enemy(50,50)
		end

		cam_update(dt)

		lights_update(dt)

		if(hitstop == 0) then
			player_update(dt)
		end

		enemies_update(dt)

		world_update(dt)

		char_update(dt)

		blood_update(dt)

		physObj_update(dt)

		lights_update(dt)

		check_hitboxs()

		spawner_update(dt)

		drawables_update(dt)
		
		decor_update(dt)
		
		tint_update(dt)
		fog_update(dt)
		
		music_control()
		
		frameCounter = frameCounter + 1
		frameCounter = frameCounter % 60
		if(frameCounter % 2 == 0) then
			trim_floors()
		end
	elseif(state == "edit") then
		editor_update(dt)
		screen_bounds()
	elseif(state == "decalSelect") then
		decal_select_update(dt)
	elseif(state == "endScreen") then
		endScreenFade = math.min(endScreenFade + dt, 1)
	end

end


-- quick demo ending logic
local hasKey = false
function get_hasKey()
	return hasKey
end

local hasOpenedDoor = false
function get_hasOpenedDoor()
	return hasOpenedDoor
end

function love.keypressed(key)
	if(key == "escape") then
		love.event.quit()
	end

	if(key == "tab") then
		if(state == "play") then editor_start() else editor_stop() end
	end

	if(state == "edit") then
		editor_keypressed(key)
	end
	
	-- key demo ending logic
	if(state == "play" and key == "e") then
		local px, py = get_playerPos()
		-- key pos 7072, 1838
		if(hasKey == false and CheckCollision(px-80, py-80, 160, 160, 7072, 1838, 32, 32)) then
			hasKey = true
			remove_key()
			add_spark(7072 + 20,1838 + 64, {1,0.9,0}, 5, 0, 20, "blood")
			play_sfx("ting2", px, py)
		end
		if(hasKey and CheckCollision(px-20, py-20, 40, 40, 2523, 2454,64,64)) then
			--state = "endScreen"
			hasOpenedDoor = true
		end
		-- door pos 2523, 2454
	end
end

function love.mousepressed(x,y,button)
	if(state == "edit") then
		editor_mousepressed(x,y,button)
	elseif(state == "decalSelect") then
		decal_select_mousepressed(x,y,button)
	end
end


local wallImg = love.graphics.newImage("wall.png")

local glowImg = love.graphics.newImage("glow.png")
glowImg:setFilter("linear","linear")

local canvas = love.graphics.newCanvas(love.graphics.getWidth(), love.graphics.getHeight())
local lightsCanvas = love.graphics.newCanvas(love.graphics.getWidth(), love.graphics.getHeight())
local gameCanvas = love.graphics.newCanvas(love.graphics.getWidth(), love.graphics.getHeight())

function resize(w,h)
	fog_resize()
	canvas = love.graphics.newCanvas(w, h)
	lightsCanvas = love.graphics.newCanvas(w, h)
	gameCanvas = love.graphics.newCanvas(w, h)
end

function set_canvas_back()
	love.graphics.setCanvas{canvas, stencil = true}
end

local font = love.graphics.getFont()
function cprint(text,x,y)
	local w = font:getWidth(text)
	local h = font:getHeight()
	love.graphics.print(text, x - w/2, y - h/2)
end

function love.draw()
	if(state == "play") then
		--love.graphics.draw(wallImg, 100, 100)	

		love.graphics.setCanvas{canvas, stencil = true}
		love.graphics.clear(0,0,0,0)
		love.graphics.setBlendMode("alpha")



		cam_set()

		world_draw()

		-- things on ground

		love.graphics.stencil(floor_stencil, "replace", 1) ------------

		love.graphics.setStencilTest("equal", 1)
		
		--local x, y = get_playerPos()
		--love.graphics.setColor(1,125/255,0,0.15)
		--love.graphics.draw(glowImg, x, y, 0, 5, 5, glowImg:getWidth()/2, glowImg:getHeight()/2)
		love.graphics.setColor(1,1,1)

		decor_draw()

		blood_drawSplats()

		lights_drawCandles()

		lights_draw2()

		char_draw()
		player_draw()

		

		physObj_draw()

		world_drawFountains()

		

		models_draw_q()

		drawables_draw()

					

		lights_drawOrbs()

		blood_draw()

		--world_shadows()

		--soft_shadow()

		love.graphics.setStencilTest()
		world2_draw()

		local mx, my = get_mousePos()
	

		love.graphics.setCanvas()
		
			
		
		cam_unset()



		love.graphics.setCanvas{lightsCanvas, stencil = true}
		
	

		
		love.graphics.setBlendMode("add")
		love.graphics.clear()
		--love.graphics.setBlendMode("add", "premultiplied")
		lights_draw()
		love.graphics.setColor(1,1,1,0.5)
		--love.graphics.rectangle("fill",0,0,1920,1080)
		love.graphics.setColor(1,1,1)
		
		love.graphics.setStencilTest()

		--love.graphics.setCanvas{gameCanvas, stencil = true}

		local px, py = get_playerPos()
		local function stencil()
			cam_set()

			world_shadows(px,py,10) -- was 6 r

			cam_unset()
		end	

		love.graphics.setBlendMode("alpha")
		love.graphics.stencil(stencil, "replace", 1)
		love.graphics.setStencilTest("equal", 1)
		love.graphics.setColor(0,0,0)
		love.graphics.rectangle("fill", 0,0, 1920, 1080)
		love.graphics.setColor(1,1,1)
		love.graphics.setStencilTest()
		
		cam_set()
		soft_shadow()
		cam_unset()
		
		love.graphics.setCanvas()
		
		love.graphics.setBlendMode("alpha","premultiplied")
		
		love.graphics.setCanvas{gameCanvas, stencil = true}
		love.graphics.clear()
		
		
		
		love.graphics.draw(lightsCanvas)
		
		--love.graphics.setStencilTest()

		love.graphics.setBlendMode("multiply","premultiplied")
		
		love.graphics.draw(canvas)
		
		love.graphics.setBlendMode("alpha", "premultiplied")
		love.graphics.setCanvas()

		love.graphics.draw(gameCanvas)



		
		--local function stencil()
			--cam_set()

			--floor_stencil()

			--cam_unset()
		--end

		
		set_fog()
		love.graphics.draw(lightsCanvas)
		love.graphics.setShader()
		love.graphics.setBlendMode("alpha")

		

		love.graphics.setStencilTest()

		cam_set()
		--lights_draw2()
		cam_unset()
		
		--[[ darken outside map fog
			
		local function stencil()
			cam_set()

			--world_shadows(px,py,15)
			floor_stencil()

			cam_unset()
		end	

		
		love.graphics.stencil(stencil, "replace", 1)
		love.graphics.setStencilTest("equal", 0)
		love.graphics.setColor(0.05,0,0.05)
		--love.graphics.rectangle("fill", 0,0, 1920, 1080)
		love.graphics.setColor(1,1,1)
		cam_set()
		world_wallTops()
		cam_unset()
		love.graphics.setStencilTest()
		
		--]]
	
		--[[ darken outside map fog
			--]]
		local depth = 50
		local step = 4
		for offset = 1, depth do	
			local count = offset
			local function stencil()
				cam_set()

				--world_shadows(px,py,15)
				world_wallTopStencil(count * step)

				cam_unset()
			end	

			
			love.graphics.stencil(stencil, "replace", 1)
			love.graphics.setStencilTest("equal", 0)
			love.graphics.setColor(0,0,0, offset/depth)
			love.graphics.rectangle("fill", 0,0, 1920, 1080)
			love.graphics.setColor(1,1,1)
		end
		love.graphics.setStencilTest()
		
		--]]
	
		love.graphics.setStencilTest()
		
		--love.graphics.setCanvas()
		--love.graphics.draw(gameCanvas)

		love.graphics.setBlendMode("alpha", "premultiplied")
		--love.graphics.setColor(1,1,1,0)
		--love.graphics.draw(lightsCanvas)
		love.graphics.setColor(1,1,1)
		love.graphics.setBlendMode("alpha")		

		-- lighten up whole screen
		tint_draw()
		love.graphics.setColor(0,0,1,0.1)
		--love.graphics.draw(canvas)
		love.graphics.setColor(1,1,1)

		

		cam_setScale()

		player_UI()

		cam_unsetScale()

		--cam_set()
		--screen_bounds()	
		--cam_unset()

		love.graphics.print(num_chars().." ,"..get_char(1).team)

		drawables_ui()

	elseif(state == "edit" or state == "decalSelect") then
		--love.graphics.push()
		--love.graphics.translate(-cam.x + love.graphics.getWidth()/2, -cam.y + love.graphics.getHeight()/2)

		cam_set()
		
		world_draw()

		decor_draw()

		lights_drawCandles()

		world_drawFountains()

		editor_draw()

		cam_unset()

		editor_ui()
	
		if(state == "decalSelect") then
			decal_select_draw()
		end
	
		--draw_grid()
	elseif(state == "endScreen") then
		love.graphics.setColor(1,1,1,endScreenFade)
	
		cam_setScale()
		cprint("End of Demo",0,0)
		cam_unsetScale()
		
		love.graphics.setColor(1,1,1)
	end


	hitbox_draw()

	love.graphics.print(love.timer.getFPS(),0,20)
end

function dist(x1,y1,x2,y2)
	return math.sqrt((x2 - x1)^2 + (y2 - y1)^2)
end

function CheckCollision(x1,y1,w1,h1, x2,y2,w2,h2)
  return x1 <= x2+w2 and
         x2 <= x1+w1 and
         y1 <= y2+h2 and
         y2 <= y1+h1
end