local models = {}

function models_load()
	new_model("models/body", "body")
	new_model("knocked/body.png", "knockedBody", 6)
	new_model("models/weapon2", "weapon2")
	new_model("head.png", "head", 8)
	new_model("models/head", "head2")
	new_model("models/shoulder", "shoulder")
	new_model("models/redShoulder", "redShoulder")
	new_model("models/weapon", "weapon", 3)
	new_model("arm.png","arm",4)
	new_model("models/leftZombArm","leftZombArm")
	new_model("models/rightZombArm","rightZombArm")
	new_model("models/leftPlayerArm","leftPlayerArm")
	new_model("models/rightPlayerArm","rightPlayerArm")
	new_model("arm2.png","arm2",4)
	new_model("knocked/zarm2.png","hand2",4)
	new_model("knocked/arm2.png","hand",4)
	new_model("models/test", "test")
	new_model("models/helmet", "helmet")
	new_model("models/redHelmet", "redHelmet")
	new_model("models/armour", "armour")
	new_model("models/redArmour", "redArmour")
	new_model("models/castle", "castle")
	new_model("models/fountain", "fountain")
	new_model("models/spike", "spike")
end

function new_model(file,model,depth)
	models[model] = {}
	local m = models[model]
	m.depth = 0

	local info = love.filesystem.getInfo(file)
	if(info.type == "directory") then
		m.imgs = {}
		m.imgDatas = {}
		local files = love.filesystem.getDirectoryItems(file)
		for i = 1, #files do
			m.imgs[i] = love.graphics.newImage(file.."/"..i..".png")
			m.imgDatas[i] = love.image.newImageData(file.."/"..i..".png")
		end

		-- rotated version
		m.rotImgs = {}
		local datas = {}

		for z = 1, m.imgs[1]:getWidth() do
			datas[z] = love.image.newImageData(#m.imgs, m.imgs[1]:getHeight())
		end

		for y = 0, m.imgs[1]:getHeight() - 1 do
			for x=0,m.imgs[1]:getWidth() - 1 do
				for z = 1, #m.imgs do
					--local img = m.imgs[z]
					local r, g, b, a = m.imgDatas[z]:getPixel(x, y)

					datas[x+1]:setPixel(z-1,y,r,g,b,a)
				end
				--local r, g, b = img:getPixel(x,y)
				--data:setPixel(z-1,y,r,g,b)

			end
		end



		for z = 1, #datas do
			m.rotImgs[z] = love.graphics.newImage(datas[z])
		end

	elseif(info.type == "file") then
		m.img = love.graphics.newImage(file)
		m.depth = depth
	end
	table.insert(models,m)
end

local models_q = {} -- queue of models to be drawn, like drawables

function model_draw(model,x,y,angle,sx,sy,ox,oy,height,rotated,transparent)
	local mq = {model = model, x = x, y = y, angle = angle, sx = sx, sy = sy, ox = ox, oy = oy, height = height, rotated = rotated, transparent = transparent}
	table.insert(models_q, mq)
end

function models_draw_q()

	for i=#models_q,1,-1 do
		local v = models_q[i]
		if(v.type == "line") then
			
		else
			if(on_screen(v.x,v.y,50,50) == false) then
				table.remove(models_q, i)
			end
		end
	end
	
	local sort = function(d1, d2)
		--return d1.height < d2.height
		
		local h1, h2 = 0,0
		
		--if(d1.model ~= "line" and d2.model ~= "line") then
			if(d1.model ~= "line" and models[d1.model].imgs) then
				h1 = #models[d1.model].imgs
				if(d1.rotated) then
					--h1 = -#models[d1.model].rotImgs
					h1 = 0
				end
			end
			if(d2.model ~= "line" and models[d2.model].imgs) then
				h2 = #models[d2.model].imgs
				if(d2.rotated) then
					--h2 = -#models[d2.model].rotImgs
					h2 = 0
				end
			end
		--end
		
		--love.graphics.circle("line", d2.x,d2.y + d2.height - h2,8) 
		
		--return d2.y - d2.height + h2 > d1.y - d1.height + h1
		


			local z1, z2 = d1.y + d1.height - h1, d2.y + d2.height - h2
			if(d1.model == "line") then z1 = (d1.y * 3 + d1.oy) / 4 + d1.height end
			if(d2.model == "line") then z2 = (d2.y * 3 + d2.oy) / 4 + d2.height end
			return z1 < z2		

		

	end

	table.sort(models_q, sort)

	love.graphics.setColor(1,1,1)
	
	for i,v in ipairs(models_q) do
		local model,x,y,angle,sx,sy,ox,oy,height,rotated,transparent = v.model, v.x, v.y, v.angle, v.sx, v.sy, v.ox, v.oy, v.height, v.rotated, v.transparent
		
		if(model == "line") then
			for i = 3,1,-1 do -- draw arm
				love.graphics.setColor(68/255 - i/3 * 0.25,68/255 - i/3 * 0.25,68/255 - i/3 * 0.25)
				love.graphics.circle("fill", x, y, 6*i/3/2)
				love.graphics.setLineWidth(6 * i/3)
				love.graphics.line(x, y, ox, oy)
				
			end		
			love.graphics.setColor(1,1,1)
		else
			local m = models[model]
			if(m.img) then
				for i=1,m.depth do
					local img = models[model].img
					--add_drawable("img", {img = img, x=x,y=y - i,angle=angle,sx=sx,sy=sy,ox=ox*img:getWidth(),oy=oy*img:getHeight(),height=i+height})
					love.graphics.draw(img,x,y - i,angle,sx,sy,ox*img:getWidth(),oy*img:getHeight())

				end
			else
				if(rotated) then
					for i=1,#m.rotImgs do
						local img = m.rotImgs[i]
						--add_drawable("img", {img = img, x=x,y=y - i +#m.rotImgs/2,angle=angle+math.pi,sx=sx,sy=sy,ox=ox*img:getWidth(),oy=oy*img:getHeight(),height=i+height-#m.rotImgs/2})
						love.graphics.draw(img,x,y - i +#m.rotImgs/2,angle+math.pi,sx,sy,ox*img:getWidth(),oy*img:getHeight())
					end
				else
					for i=1,#m.imgs do
						local img = m.imgs[i]
						--add_drawable("img", {img = img, x=x,y=y - i,angle=angle,sx=sx,sy=sy,ox=ox*img:getWidth(),oy=oy*img:getHeight(),height=i+height})
						
						local alpha = 1
						if(transparent) then
							alpha = transparent
						end
						love.graphics.setColor(1,1,1,alpha)
						love.graphics.draw(img,x,y - i,angle,sx,sy,ox*img:getWidth(),oy*img:getHeight())
						
						
					end
					love.graphics.setColor(1,1,1)
				end
				
				--love.graphics.circle("line",x,y,2)
			end
		end
	end
	love.graphics.setColor(1,1,1)
	models_q = {}
end