

local physObj = {}

function add_physObj(model, x, y, z, angle, xvel, yvel, zvel)
	local p = {model = model, x = x, y = y, z = z, angle = angle, xvel = xvel, yvel = yvel, zvel = zvel,rotated = false,life = 30}
	table.insert(physObj, p)
end

function num_physObj()
	return #physObj
end

function get_physObj(i)
	return physObj[i]
end

local gravity = 500
local friction = 100

-- make world bounce col

function physObj_update(dt)
	--for i,v in ipairs(physObj) do
	for i=#physObj,1,-1 do
		local v = physObj[i]

		local dx, dy = 0, 0
		if(v.xvel ~= 0 and v.yvel ~= 0 and v.zvel ~= 0) then
			local a = math.atan2(v.yvel, v.xvel)
			dx, dy = math.cos(a) * friction, math.sin(a) * friction
		end

		v.xvel = v.xvel - dx * dt
		v.yvel = v.yvel - dy * dt

		v.zvel = v.zvel - gravity * dt

		if(v.model == "body" and v.z <= 10) then v.rotated = true end

		if(v.z + v.zvel * dt < 0) then
		
			
		
			if(v.z > 0) then
				add_spark(v.x, v.y - 10 , {0.4,0.4,0.4}, 7, 0, 30, "dust")
				play_sfx("bodyImpact",v.x,v.y)
			end
			
			v.z = 0
			v.zvel = -v.zvel * 0.75
			v.xvel = v.xvel * 0.9 -- was 0.75
			v.yvel = v.yvel * 0.9

			if(math.abs(v.zvel) < 100) then
				v.zvel = 0
				v.xvel = 0
				v.yvel = 0
			end
		end
		v.angle = v.angle + v.xvel/50 * dt
		local prevXvel, prevYvel = v.xvel, v.yvel
		v.x, v.y, v.xvel, v.yvel = world_bounceCol(v.x, v.y, 20, 20, v.xvel, v.yvel, dt)
		if(math.abs(v.xvel - prevXvel) > 10 or math.abs(v.yvel - prevYvel) > 10) then
			if(math.abs(v.xvel) > 100 or math.abs(v.yvel) > 100) then
				play_sfx("pushup", v.x, v.y)
				
			end
		end

		v.x = v.x + v.xvel * dt
		v.y = v.y + v.yvel * dt
		v.z = v.z + v.zvel * dt

		-- knock back chars if body
		--if(v.model == "body" and v.rotated == false) then
		if(v.z > 20) then
			for i = 1, num_chars() do
				local char = get_char(i)
				if(char.team == 2 and char.active and char.knocked == false and CheckCollision(v.x - 10,v.y - 10, 20, 20, char.x - 10, char.y - 10, 20, 20)) then
					play_sfx("pushup", v.x, v.y)
					play_sfx("smash2", v.x, v.y)
					
					char.attackTimer = 0
					add_blood(char.x, char.y, 10,v.angle + math.pi)
					char.hp = char.hp - 1
					char.knocked = true
					char.xvel = v.xvel * 0.75
					char.yvel = v.yvel * 0.75
					char.z = char.height / 2
					char.zvel = math.random(80,100)
					v.zvel = v.zvel * 0.5
					v.xvel = v.xvel * 0.25 -- k
					v.yvel = v.yvel * 0.25					
				end	
			end
		end
		--end
		if not(on_screen(v.x-50,v.y-50,100,100)) then
			v.life = v.life - dt
			if(v.life < dt) then
				table.remove(physObj, i)
			end
		else
			v.life = math.min(10,v.life)
		end

		if not(on_ground(v.x,v.y)) then
			table.remove(physObj, i)
		end

	end
end

local shadowImg = love.graphics.newImage("shadow.png")

function physObj_draw()
	for i,v in ipairs(physObj) do
		if(on_screen(v.x - 50,v.y - 50, 100, 100)) then

			model_draw(v.model, v.x, v.y - v.z, v.angle, 1, 1, 0.5, 0.5,v.z,v.rotated)
			love.graphics.setColor(0,0,0,0.3)
			love.graphics.draw(shadowImg, v.x, v.y, 0, 1, 1, shadowImg:getWidth()/2, shadowImg:getHeight()/2)
			love.graphics.setColor(1,1,1)
		end
	end
end