local player = {}
player.char = 1
player.x = 100
player.y = 100

player.angle = 0

player.xvel = 0
player.yvel = 0

player.moveAngle = 0

player.alive = true

local moveSpeed = 100

local fadeIn = 1
player.spawnX = 0
player.spawnY = 0
function player_setSpawn(x,y)
	player.spawnX, player.spawnY = x, y
end
function player_setup()
	fadeIn = 1
	player.alive = true
	new_char() -- create player char

	set_charWeapon(1,1)

	local sx, sy = get_start()
	set_camPos(sx, sy)
	set_charPos(1,sx, sy + 150)
	set_charMaxHp(1,5)

	--[[
	local char = get_char(new_char())
	char.weapon = 1
	char.team = 1
	]]--
end

local maxLantern = 100

function player_respawn()
	chars_reset()
	player_setup()
	world_reset()
	reset_lights(true)
	local sx, sy = player.spawnX, player.spawnY
	player.x = sx
	player.y = sy
	set_camPos(sx, sy)
	set_charPos(1,sx, sy)
	set_charMaxHp(1,5)
	player.alive = true

	player.lantern = maxLantern
end

local deathTime = 10
local deathTimer = 0

player.safe = false 

function player_isSafe()
	return player.safe
end

function player_makeSafe()
	player.safe = true
end

function player_isAlive()
	return player.alive
end


local lanternDrain = 2
player.lantern = 100
player.atCandle = false

function player_getLantern()
	return player.lantern/maxLantern
end

function player_addLantern(amount)
	player.lantern = math.min(maxLantern, player.lantern + amount)
	player.atCandle = true
end

function player_update(dt)
	player.x, player.y = get_charPos(1)

	player.safe = false

	if(get_hasOpenedDoor()) then
		fadeIn = math.min(1, fadeIn + 0.75 * dt)
		if(fadeIn == 1) then
			state = "endScreen"
		end
	else
		fadeIn = math.max(0, fadeIn - 0.75 * dt)
	end
	
	if(love.mouse.isDown(1)) then
		attack_char(1)
		
		--anim_char(1, 1)
	end

	player.xvel, player.yvel = 0, 0

	if(love.keyboard.isDown("a")) then
		player.xvel = -moveSpeed
	elseif(love.keyboard.isDown("d")) then
		player.xvel = moveSpeed
	end

	if(love.keyboard.isDown("w")) then
		player.yvel = -moveSpeed
	elseif(love.keyboard.isDown("s")) then
		player.yvel = moveSpeed
	end

	local mx, my = get_mousePos()
	player.angle = math.atan2(player.y - my, player.x - mx) + math.pi

	player.moveAngle = math.atan2(player.yvel, player.xvel)

	move_char(player.char, player.angle, player.moveAngle, player.xvel, player.yvel)

	local hp = get_charHp(1)
	if(hp <= 0) then
		player.alive = false
	end

	-- complete lvl
	local ex, ey = get_exit()
	if(CheckCollision(ex-10,ey-10,20,20,player.x-25, player.y-25, 50, 50)) then
		load_lvl()
	end

	-- restart after death
	if(player.alive == false) then
		if(deathTimer >= deathTime or (love.mouse.isDown(1) and deathTimer >= 1)) then
			deathTimer = 0
			player_respawn()
		else
			deathTimer = deathTimer + 10 * dt
		end
	end

	if(isAtFountain() == true) then
		add_charHp(1,dt)
		player.lantern = math.min(maxLantern, player.lantern + dt * 10)
	elseif(player.atCandle == false) then
		player.lantern = math.max(0, player.lantern - lanternDrain * (player.lantern/maxLantern) * dt)
	end
	
	player.atCandle = false
end

function get_playerPos()
	return get_charPos(1)
end

function player_draw()

	--char_draw(player.x, player.y, player.angle, player.moveAngle)

	--love.graphics.circle("fill", player.x, player.y, 4)
end

function player_UI()
	local maxHp = get_charMaxHp(1)
	local hp = get_charHp(1)
	local hpWidth = 300
	local hpHeight = 6
	local border = 2
	local h = love.graphics.getHeight() / math.min(love.graphics.getWidth()/640, love.graphics.getHeight()/480) / 2
	local w = love.graphics.getWidth() / math.min(love.graphics.getWidth()/640, love.graphics.getHeight()/480) / 2
	--love.graphics.print(h)
	love.graphics.setColor(0.2,0,0,0.9)
	love.graphics.rectangle("fill",-w + 110 - border,h-hpHeight - 18 - border,hpWidth + border * 2,hpHeight + border * 2)
	love.graphics.setColor(0.5,0,0,0.9)
	love.graphics.rectangle("fill",-w + 110,h-hpHeight - 18,hpWidth*hp/maxHp,hpHeight)
	love.graphics.setColor(1,1,1)
	

	local lanternWidth = 300
	local lanternHeight = 6
	local border = 2

	--love.graphics.print(h)
	love.graphics.setColor(120/255, 47/255, 2/255, 0.9)
	love.graphics.rectangle("fill",-w + 110 - border,h-lanternHeight - 18 - border - lanternHeight - 6,lanternWidth + border * 2,lanternHeight + border * 2)
	love.graphics.setColor(184/255,70/255,0,0.9)
	love.graphics.rectangle("fill",-w + 110,h-lanternHeight - 18 - lanternHeight - 6,lanternWidth*player.lantern/maxLantern,lanternHeight)
	love.graphics.setColor(1,1,1)	

	-- mini map
	local minimapSize = 50
	local minimapX = -w + minimapSize + 12
	local minimapY = h - minimapSize - 12

	love.graphics.push()

	love.graphics.translate(minimapX, minimapY)

	local stencil = function()
		love.graphics.circle("fill",0,0,minimapSize)
	end
	love.graphics.stencil(stencil,"replace",1)
	love.graphics.setStencilTest("equal",1)



	local px, py = get_playerPos()
	local scale = minimapSize/50 * 0.035
	for i=1, num_floors() do
		local x, y, w, h = get_floor(i)
		local canvas = get_floorCanvas(i)

		love.graphics.draw(canvas,x*scale - px*scale,y*scale -py*scale,0,scale,scale)
		love.graphics.draw(canvas,x*scale - px*scale + 1,y*scale -py*scale + 1,0,scale,scale)
		love.graphics.draw(canvas,x*scale - px*scale + 1,y*scale -py*scale - 1,0,scale,scale)
		love.graphics.draw(canvas,x*scale - px*scale - 1,y*scale -py*scale + 1,0,scale,scale)
		love.graphics.draw(canvas,x*scale - px*scale - 1,y*scale -py*scale - 1,0,scale,scale)
		love.graphics.setColor(1,1,1,0.05)
		--love.graphics.rectangle("fill",x*scale - px*scale,y*scale - py*scale,w*scale,h*scale)
		love.graphics.setColor(1,1,1)

	end

	love.graphics.setColor(113/255,97/255,81/255,0.25)
	love.graphics.circle("fill",0,0,minimapSize)
	

	love.graphics.setColor(0,0,0,0.5)
	love.graphics.setColor(113/255/2,97/255/2,81/255/2,0.75)
	love.graphics.setColor(32/255*2,0,0,1)
	love.graphics.setColor(113/255,56/255,0/255,0.75)
	love.graphics.circle("fill",0,0,4)
	love.graphics.setColor(113/255/2,56/255/2,0/255,1)
	love.graphics.circle("fill",0,0,3)
	love.graphics.setColor(113/255,56/255,0/255,0.75)
	love.graphics.line(0,-minimapSize,0,minimapSize)
	love.graphics.line(-minimapSize,0,minimapSize,0)
	

	local rings = 4
	for j=1,rings do
		love.graphics.circle("line",0,0,j/rings*minimapSize)
	end

	love.graphics.setColor(1,1,1)

	--love.graphics.setLineWidth(16)
	--love.graphics.setColor(0/255,0/255,0/255,0.9)
	--love.graphics.circle("line",0,0,minimapSize)
	local rings = minimapSize
	for i=1,rings do
		love.graphics.setColor(0,0,0,1-(i/rings)*2)
		love.graphics.circle("line",0,0,minimapSize-i-4)
	end
	love.graphics.setColor(1,1,1)

	love.graphics.setLineWidth(8)
	love.graphics.setColor(32/255,0,65/255,0.5)
	love.graphics.circle("line",0,0,minimapSize)
	love.graphics.circle("line",0,0,minimapSize-4)
	love.graphics.setLineWidth(2)
	love.graphics.setColor(32/255/2,0,65/255/2)
	love.graphics.circle("line",0,0,minimapSize)
	love.graphics.setColor(1,1,1)
	--[[
	love.graphics.setColor(113/255/2,97/255/2,81/255/2,1)
	love.graphics.circle("line",0,0,minimapSize)
	love.graphics.setColor(113/255/2,97/255/2,81/255/2,0.75)
	love.graphics.circle("line",0,0,minimapSize-8)
	love.graphics.setLineWidth(1)
	love.graphics.setColor(1,1,1)

	--]]
	love.graphics.setColor(32/255,0,65/255)
	--love.graphics.circle("fill",0,0,4)
	love.graphics.setColor(1,1,1)
	love.graphics.setStencilTest()

	love.graphics.pop()

	-- fadeIn
	love.graphics.setColor(0,0,0,fadeIn)
	love.graphics.rectangle("fill",-w,-h,w*2,h*2)
	love.graphics.setColor(1,1,1)

	if(player_isSafe()) then
		love.graphics.print("SAFE", 0,0)
	end
end