love.audio.setDistanceModel("inverseclamped")

local sfxs = {}
--sfxs["footstep"] = love.audio.newSource("sfx/footstep_2.ogg", "static")

function add_sfx(name, file, directional)
	local sfx = {}
	sfx.directional = directional
	sfx.audio = love.audio.newSource(file, "static")
	sfx.copies = {sfx.audio}
	
	sfxs[name] = sfx
	--table.insert(sfxs, sfx)
end

add_sfx("footstep", "sfx/footstep_3.ogg", true)
add_sfx("player_footstep", "sfx/player_footstep.ogg", false)
add_sfx("swing", "sfx/swing.ogg", false)
add_sfx("bodyImpact", "sfx/bodyImpact.ogg", true)
add_sfx("pushup", "sfx/pushup.ogg", true)
add_sfx("grunt", "sfx/grunt_2.ogg", true)
add_sfx("smash", "sfx/smash.ogg", true)
add_sfx("smash2", "sfx/smash_2.ogg", true)
add_sfx("splat", "sfx/splat.ogg", true)
add_sfx("blowOut", "sfx/blowOut2.ogg",true)
add_sfx("ting", "sfx/ting.ogg",true)
add_sfx("ting2", "sfx/ting2.ogg",true)
add_sfx("switch", "sfx/switch.ogg",true)

local music = {
	love.audio.newSource("music/track1.wav", "stream")
}

local sfxVolume = 0.25 -- 0.25 --1
local musicVolume = 1
local sfxMute = false
local musicMute = false


function skip_track(t)
	for i=1,#music do
		local m = music[i]
		if(m:isPlaying()) then
			if(m:getDuration() > m:tell() + t) then
				m:seek(m:tell() + t)
			else
				m:stop()
			end
		end
	end
	for i=1,#sfxs do
		local m = sfxs[i]
		if(m:isPlaying()) then
			if(m:getDuration() > m:tell() + t) then
				m:seek(m:tell() + t)
			else
				m:stop()
			end
		end
	end
end

function applyMusicVolume()
	for i = 1, #music do
		music[i]:setVolume(musicVolume)
	end
end

function voicePlaying()
	for i = 1, #music do
		music[i]:setVolume(musicVolume * 0.7)
	end	
end

function applySfxVolume()
	for i = 1, #sfxs do
		sfxs[i].audio:setVolume(sfxVolume)
		for j=1,#sfxs[i].copies do
			sfxs[i].copies[j]:setVolume(sfxVolume)
		end
	end
end



function deSfxVolume()
	sfxVolume = math.max(0, sfxVolume - 0.1)
	applySfxVolume()
end

function incSfxVolume()
	sfxVolume = math.min(1, sfxVolume + 0.1)
	applySfxVolume()
end

function deMusicVolume()
	musicVolume = math.max(0, musicVolume - 0.1)
	applyMusicVolume()
end

function incMusicVolume()
	musicVolume = math.min(1, musicVolume + 0.1)
	applyMusicVolume()
end

function get_sfxVolume()
	return sfxVolume
end

function get_musicVolume()
	return musicVolume
end

function set_sfxVolume(x)
	sfxVolume = x
	applySfxVolume()
end

function set_musicVolume(x)
	musicVolume = x
	applyMusicVolume()
end

function set_sfxMute(b)
	sfxMute = b
end

function set_musicMute(b)
	musicMute = b
end

function get_sfxMute()
	return sfxMute
end

function get_musicMute()
	return musicMute
end

local audioPosScale = 350 --50

function sfx_setListenerPosition(x,y)
	love.audio.setPosition(x/audioPosScale,y/audioPosScale)
end

function play_sfx(i,x,y)
	if(sfxMute == false) then
		local sfx
		local directional = sfxs[i].directional
		--love.audio.stop(sfxs[i])
		local r = 1 + math.random(-5,5)/100
		for j=1,#sfxs[i].copies do
			if(sfxs[i].copies[j]:isPlaying() and j == #sfxs[i].copies) then
				sfxs[i].copies[j + 1] = sfxs[i].audio:clone()
				sfx = sfxs[i].copies[j + 1]
				--sfxs[i].copies[j + 1]:setPitch(r)
				--love.audio.play(sfxs[i].copies[j + 1])
			elseif(sfxs[i].copies[j]:isPlaying() == false) then
				sfx = sfxs[i].copies[j]
				--sfxs[i].copies[j]:setPitch(r)
				--love.audio.play(sfxs[i].copies[j])
				break
			end		
		end

		if(directional) then
			sfx:setPosition(x/audioPosScale,y/audioPosScale)
		end
		sfx:setVolume(sfxVolume)
		sfx:setPitch(r)
		love.audio.play(sfx)
	end
end



for i=1,#music do
	music[i]:setLooping(true)
end

function stop_music()
	for i=1,#music do
		music[i]:stop()
	end
end

function sfxs_stop()
	for i=1,#sfxs do
		sfxs[i]:stop()
	end	
end

function stop_sfx(i)
	sfxs[i]:stop()
end

function change_music(m)
	for i=1,#music do
		if(i == m) then
			if(music[i]:isPlaying() == false) then
				music[i]:play()
			end
		else
			if(music[i]:isPlaying()) then
				music[i]:stop()
			end			
		end
	end
end

function music_control()
	change_music(1)
end

