
local start = {}
start.x = 100
start.y = 100


local exit = {}
exit.x = 200
exit.y = 200

function get_exit()
	return exit.x, exit.y
end

function set_exit(x,y)
	exit.x, exit.y = x, y
end

function get_start()
	return start.x, start.y
end

function set_start(x,y)
	start.x, start.y = x, y
end

local fountains = {}
function add_fountain(x,y)
	local f= {x=x,y=y,active = false,lightFadeIn=0}
	table.insert(fountains,f)
end

function remove_fountain(x,y)
	for i,v in ipairs(fountains) do
		if(dist(x,y,v.x,v.y) < 20) then
			table.remove(fountains, i)
			break
		end
	end
end

function num_fountains()
	return #fountains
end

function get_fountain(i)
	return fountains[i].x, fountains[i].y, fountains[i].lightFadeIn, fountains[i].active
end


local gates = {}

function add_gate(x,y,w,h,id)
	local g = {}
	g.x = x
	g.y = y
	g.w = w
	g.h = h
	g.closed = true
	g.id = id
	g.timer = 1
	table.insert(gates, g)
end

function num_gates()
	return #gates
end

function remove_gate(i)
	if(gates[i]) then
		table.remove(gates, i)
	end
end

function get_gate(i)
	if not(gates[i]) then return 0,0,0,0,"" else
		return gates[i].x, gates[i].y, gates[i].w, gates[i].h, gates[i].id
	end
end

function move_gate(i, x,y)
	if(gates[i]) then
		gates[i].x = gates[i].x + x
		gates[i].y = gates[i].y + y
	end
end

function resize_gate(i, w, h)
	if(i > 0 and i <= #gates) then
		local f = gates[i]
		f.w = f.w + w
		f.h = f.h + h
	end	
end

function get_gate_atPos(x,y)
	for i,v in ipairs(gates) do
		if(CheckCollision(x,y,1,1,v.x,v.y,v.w,v.h)) then
			return i
		end
	end
	return 0
end

local switches = {}
local switchRange = 80

function num_switches()
	return #switches
end

function get_switch(i)
	if(switches[i]) then
		return switches[i].x, switches[i].y, switches[i].id
	else
		return 0,0,""
	end
end

function add_switch(x,y,id)
	local s = {}
	s.x = x
	s.y = y
	s.id = id
	s.pressed = false
	table.insert(switches, s)
end

function remove_switch(x,y)
	for i,v in ipairs(switches) do
		if(dist(x,y,v.x,v.y) < 17) then
			table.remove(switches, i)
			break
		end
	end
end

local floors = {}

function add_floor(x,y,w,h)
	local f = {}
	f.x = x
	f.y = y
	f.w = w
	f.h = h
	f.wallReveal = 0
	f.hasFountain = false

	f.canvas = love.graphics.newCanvas(w,h)

	table.insert(floors,f)
end

local floorsCopy = {}

function add_floorsCopy(x, y, w, h)
	local fc = {x=x, y=y, w=w, h=h}
	table.insert(floorsCopy, fc)
end

function copy_floors()
	for i,v in ipairs(floors) do
		add_floorsCopy(v.x, v.y, v.w, v.h)
	end
end

function load_all_floors()
	for i,v in ipairs(floorsCopy) do
		add_floor(v.x, v.y, v.w, v.h)
		for j,k in ipairs(fountains) do
			if(CheckCollision(v.x,v.y,v.w,v.h,k.x,k.y,1,1)) then
				floors[#floors].hasFountain = true
			end
		end			
	end
	
	floorsCopy = {}
end

local trimDist = 850
function trim_floors()
	--[[
	for i = #floors, 25, -1 do
		table.remove(floors, i)
	end
	]]--
	
	local px, py = get_playerPos()
	for i=#floors,1,-1 do
		local v = floors[i]
		--if(math.abs(v.x + v.w/2 - px) > trimDist and math.abs(v.y + v.h/2 - py) > trimDist) then
		if(CheckCollision(px - trimDist/2,py - trimDist/2,trimDist,trimDist, v.x, v.y, v.w, v.h) == false) then
			add_floorsCopy(v.x, v.y, v.w, v.h)
			table.remove(floors, i)
		end
	end
	
	for i=#floorsCopy,1,-1 do
		local v = floorsCopy[i]
		--if(math.abs(v.x - px) < trimDist and math.abs(v.y - py) < trimDist) then
		if(CheckCollision(px - trimDist/2,py - trimDist/2,trimDist,trimDist, v.x, v.y, v.w, v.h)) then
			add_floor(v.x, v.y, v.w, v.h)
			for j,k in ipairs(fountains) do
				if(CheckCollision(v.x,v.y,v.w,v.h,k.x,k.y,1,1)) then
					floors[#floors].hasFountain = true
				end
			end		
			table.remove(floorsCopy, i)
		end		
	end
end

function num_floors()
	return #floors
end

function get_floorCanvas(i)
	return floors[i].canvas
end

function get_floor(i)
	if(i > 0 and i <= #floors) then
		local f = floors[i]
		return f.x, f.y, f.w, f.h
	end
	return 0,0,0,0
end

function move_floor(i, x, y)
	if(i > 0 and i <= #floors) then
		local f = floors[i]
		f.x = f.x + x
		f.y = f.y + y
	end
end

function resize_floor(i, w, h)
	if(i > 0 and i <= #floors) then
		local f = floors[i]
		f.w = f.w + w
		f.h = f.h + h
	end	
end

function remove_floor(i)
	if(i > 0 and i <= #floors) then
		table.remove(floors,i)
	end	
end

function get_floor_atPos(x,y)
	for i,v in ipairs(floors) do
		if(CheckCollision(x,y,1,1,v.x,v.y,v.w,v.h)) then
			return i
		end
	end
	return 0
end



function light_floors()
	reset_lights()

	for i,v in ipairs(floors) do
		if(v.hasFountain == false) then
			local r = math.max(1, v.w * v.h / 100000)
			local r = math.max(0, v.w * v.h / 80000)
			for j=1,r do
				local x, y = math.random(v.x, v.x+v.w), math.random(v.y, v.y+v.h)
				add_light(x,y, math.random(20,40)/10,{math.random(100,255)/255,math.random(100,255)/255, math.random(100,255)/255})
			end
		end
	end
	for i,v in ipairs(floorsCopy) do
		local canPlace = true
		for j,k in ipairs(fountains) do
			if(CheckCollision(v.x,v.y,v.w,v.h,k.x,k.y,1,1)) then
				canPlace = false
			end
		end
		if(canPlace) then
			local r = math.max(1, v.w * v.h / 100000)
			local r = math.max(0, v.w * v.h / 80000)
			for j=1,r do
				local x, y = math.random(v.x, v.x+v.w), math.random(v.y, v.y+v.h)
				add_light(x,y, math.random(20,40)/10,{math.random(100,255)/255,math.random(100,255)/255, math.random(100,255)/255})
			end			
		end
	end
	for i,v in ipairs(fountains) do
		--add_light(start.x,start.y, 5,{0.5,0,0})
		--add_light(v.x,v.y, 5,{182/255,1,235/255})
	end
end

function on_ground(x,y)
	for i,v in ipairs(floors) do
		if(CheckCollision(v.x,v.y+20,v.w,v.h,x,y,1,1)) then
			return true
		end
	end
	return false
end

function can_spawn(x,y)
	for i,v in ipairs(floors) do
		if(CheckCollision(v.x,v.y+20,v.w,v.h,x,y,1,1) and v.hasFountain == false) then
			return true
		end
	end
	return false
end

local wallHeight = 100

function on_ground2(x,y) -- no body width, not phys obj
	for i,v in ipairs(floors) do
		if(CheckCollision(v.x,v.y - wallHeight,v.w,v.h+wallHeight,x,y,1,1)) then
			return true
		end
	end
	return false
end

function world_col(x,y, w, h, dx,dy,dt)

	local onGroundX = false
	local leftOnGround = false
	local rightOnGround = false
	local onGroundY = false
	for i,v in ipairs(floors) do
		if(CheckCollision(v.x,v.y + 20,v.w,v.h,x + dx * dt,y,1,1)) then
			onGroundX = true
		end

		if(CheckCollision(v.x,v.y + 20,v.w,v.h,x + dx * dt - 15,y,1,1)) then
			leftOnGround = true
		end

		if(CheckCollision(v.x,v.y + 20,v.w,v.h,x + dx * dt + 15,y,1,1)) then
			rightOnGround = true
		end

		if(CheckCollision(v.x,v.y + 20,v.w,v.h,x ,y + dy * dt,1,1)) then
			onGroundY = true
		end
	end

	if(onGroundX == true and (leftOnGround == false or rightOnGround == false))then 
		onGroundX = false

	end

	for i,v in ipairs(gates) do
		if(v.closed == true) then
			if(y > v.y and y < v.y + v.h) then
				if(x < v.x and x + dx * dt > v.x) then
					onGroundX = false
				elseif(x > v.x + v.w and x + dx * dt < v.x + v.w) then
					onGroundX = false
				end
			end
			if(x > v.x and x < v.x + v.w) then
				if(y < v.y and y + dy * dt > v.y) then
					onGroundY = false
				elseif(y > v.y + v.h and y + dy * dt < v.y + v.h) then
					onGroundY = false
				end
			end			
		end
	end
	
	local fountainR = 120
	for i,v in ipairs(fountains) do
		if(CheckCollision(v.x - fountainR, v.y - fountainR, fountainR*2, fountainR*2, x + dx * dt, y, w, h)) then
			if(dist(v.x,v.y,x + dx*dt,y) < fountainR) then
				onGroundX = false
				dx = -dx
			end
		end
		if(CheckCollision(v.x - fountainR, v.y - fountainR, fountainR*2, fountainR*2, x, y + dy * dt, w, h)) then
			if(dist(v.x,v.y,x, y + dy * dt) < fountainR) then
				onGroundY = false
				dy = -dy
			end
		end
	end

	if(onGroundY == false) then 
		dy = 0
	end

	if(onGroundX == false) then
		dx = 0
		if(leftOnGround == false) then
			dx = 100
		elseif(rightOnGround == false) then
			dx = -100
		end
	end

	return x, y, dx, dy

end

function world_bounceCol(x, y, w, h, dx, dy, dt)
	local onGroundX = false
	local onGroundY = false
	for i,v in ipairs(floors) do
		if(CheckCollision(v.x,v.y + 20,v.w,v.h,x + dx * dt,y,1,1)) then
			onGroundX = true
		end

		if(CheckCollision(v.x,v.y + 20,v.w,v.h,x ,y + dy * dt,1,1)) then
			onGroundY = true
		end
	end

	if(onGroundY == false) then 
		dy = -dy
	end

	if(onGroundX == false) then
		dx = -dx
	end

	return x, y, dx, dy
end

function world_bounceCol2(x, y, w, h, dx, dy, dt)
	local onGroundX = false
	local onGroundY = false
	for i,v in ipairs(floors) do
		if(CheckCollision(v.x,v.y,v.w,v.h,x + dx * dt,y,1,1)) then
			onGroundX = true
		end

		if(CheckCollision(v.x,v.y,v.w,v.h,x ,y + dy * dt,1,1)) then
			onGroundY = true
		end
	end

	if(onGroundY == false) then 
		dy = -dy
	end

	if(onGroundX == false) then
		dx = -dx
	end

	return x, y, dx, dy
end

local floorFadeOut = 0
local atFountain = true
function isAtFountain()
	return atFountain
end

function world_update(dt)
	local px, py = get_playerPos()
	

	for i,v in ipairs(gates) do
		if(v.timer > 0 and v.closed == false) then
			v.timer = math.max(0, v.timer - dt)
		end
	end
	for i,v in ipairs(switches) do
		if(v.pressed == false and love.keyboard.isDown("e") and CheckCollision(v.x - switchRange/2,v.y - switchRange/2, switchRange, switchRange, px, py, 1,1)) then
			v.pressed = true
			add_spark(v.x, v.y - 14, {0.6,0.6,0.6}, 8, 0, 20, "blood")
			add_spark(v.x, v.y - 14, {0.5,0.5,0.5}, 8, 0, 20, "blood")
			play_sfx("switch", v.x, v.y)
			for j,k in ipairs(gates) do
				if(k.id == v.id) then
					k.closed = false
					for n = 0,k.w/32 - 1 do
						for m = 0, k.h/32 - 1 do
							add_spark(k.x + n * 32 + 16,k.y + 8 + m * 32, {0.6,0.6,0.6}, 10, 0, 20, "blood")
							add_spark(k.x + n * 32 + 16,k.y + 8 + m * 32, {0.5,0.5,0.5}, 10, 0, 20, "blood")
						end
					end
				end
			end
		end
	end

	atFountain = false
	local inRoomWithFountain = false
	if(player_isAlive() == false) then
		floorFadeOut = math.min(0.8, floorFadeOut + 2 * dt)
	end
	local px, py = get_playerPos()
	for i,v in ipairs(floors) do
		if(v.hasFountain and CheckCollision(v.x,v.y,v.w,v.h,px,py,1,1)) then
			inRoomWithFountain = true
		end
		if(v.y < py) then
			v.wallReveal = math.min(1, v.wallReveal + 2 * dt)
		else
			v.wallReveal = 0
		end
	end
	for i,v in ipairs(fountains) do
		if(v.active == false) then
			if(CheckCollision(px,py,1,1,v.x-150,v.y-150,300,300)) then
				v.active = true
				--light_floors()
				reset_lights()
			end
			v.lightFadeIn = math.max(0, v.lightFadeIn - dt)
		else
			--[[
			if not(CheckCollision(px,py,1,1,v.x-300,v.y-300,600,600)) then -------------------------------
				v.active = false

			end
			]]
			
			if(inRoomWithFountain == false) then
				v.active = false
			end

			player_setSpawn(v.x,v.y+150)
			v.lightFadeIn = math.min(1, v.lightFadeIn + dt)
			atFountain = true
		end

	end
end

local wallTopWidth = 20

function world2_draw()
	for i,v in ipairs(floors) do

	end
end

function floor_stencil()
	for i,v in ipairs(floors) do
		love.graphics.rectangle("fill", v.x, v.y - wallHeight,v.w,v.h + wallHeight)
	end
end

local floorTile = love.graphics.newImage("floorTile.png")
local floorTile2 = love.graphics.newImage("floorTile2.png")
local wallTile = love.graphics.newImage("wallTile2.png")
local wallTile2 = love.graphics.newImage("wallTile.png")

--local floorTile = love.graphics.newImage("tile3.png")

local points = {}
local edges = {}
local function add_edge(x1,y1,x2,y2)
	edges[#edges+1] = {x1=x1,y1=y1,x2=x2,y2=y2}
end
local function add_point(x,y)
	points[#points+1] = {x=x,y=y,ylen = 0,xlen=0}
end

function world_createMesh()
	local px,py = get_playerPos()
	edges = {}
	points = {}
	for i,v in ipairs(floors) do
		if(on_screen(v.x,v.y,v.w,v.h)) then
			for j=1,4 do -- create points
				local x, y = 0, 0
				--[[
				if(1 == 1 or py > v.y) then -------------------------------------------------------- test always using wallheight, then always using both, should remove edges on floor anyway
					if(j == 1) then x, y = v.x, v.y - wallHeight end
					if(j == 2) then x, y = v.x + v.w, v.y - wallHeight end
				else
					if(j == 1) then x, y = v.x, v.y end
					if(j == 2) then x, y = v.x + v.w, v.y end
				end
				]]
				
				if(j == 1) then x, y = v.x, v.y end
				if(j == 2) then x, y = v.x + v.w, v.y end
				if(j == 3) then x, y = v.x + v.w, v.y + v.h end
				if(j == 4) then x, y = v.x, v.y + v.h end
				--if(j == 5) then x, y = v.x, v.y - wallHeight end
				--if(j == 6) then x, y = v.x + v.w, v.y - wallHeight end

				add_point(x,y)
				--add_point(x+dx,y+dy)
				--love.graphics.line(x,y,x+dx,y+dy)
			end
		end


		--if(v.y < py) then -- 1
			--love.graphics.circle("fill",v.x,v.y-wallHeight,4)
			add_point(v.x,v.y-wallHeight)
			--love.graphics.circle("fill",v.x+v.w,v.y-wallHeight,4)
			add_point(v.x + v.w,v.y-wallHeight)
		--end -- 2

		--if(v.y+v.h < py) then -- 3
			add_point(v.x,v.y+v.h-wallHeight)
			add_point(v.x+v.w,v.y+v.h-wallHeight)
		--end -- 4

		-- midpoints
		add_point(v.x+v.w/2,v.y)
		add_point(v.x+v.w,v.y+v.h/2)
		add_point(v.x+v.w/2,v.y)
		add_point(v.x,v.y+v.h/2)
	end

	-- idea is to only light floor then use shader to light walls based on light below on floor, right now only lighting top

	-- vertical edges ----------------

	for i=1,#points do
		local p1 = points[i]

		for j=1,#points do
			if(i ~= j) then
				local p2 = points[j]

				-- connect verticals
				if(p1.x == p2.x and p2.y > p1.y) then
					local len = p2.y - p1.y
					if(p1.ylen == 0 or len < p1.ylen) then
						-- remove last edge if this edge ylen not 0, edge already been added
						-- another point adding to same 2nd point
						if(p1.ylen ~= 0) then
							table.remove(edges,#edges)
						end
						p1.ylen = len
						p2.connectedy = true
						add_edge(p1.x,p1.y,p2.x,p2.y) -- edge already added reason for duplicates, adds everytime finds shorter
					end
				end

			end
		end

		--love.graphics.circle("fill",p1.x,p1.y,4)
	end

	-- remove edges off ground

	for i=#edges,1,-1 do
		local e = edges[i]
		if(on_ground2((e.x1+e.x2)/2,(e.y1+e.y2)/2) == false) then
			table.remove(edges,i)
		end
	end

	-- remove edges fully on ground
	for i=#edges,1,-1 do
		local e = edges[i]
		if(on_ground2(e.x1 - 10, e.y1+10) == true and on_ground2(e.x1 + 10, e.y1+10) == true) then
			--love.graphics.rectangle("fill",e.x1 - 10, e.y1+10,1,1)
			--love.graphics.rectangle("fill",e.x1 + 10, e.y1+10,1,1)
			table.remove(edges,i)
		end
	end

	---------------------------

	-- horizontal edges

	for i=1,#points do
		local p1 = points[i]

		for j=1,#points do
			if(i ~= j) then
				local p2 = points[j]

				-- connect verticals
				if(p1.y == p2.y and p2.x > p1.x) then
					local len = p2.x - p1.x
					if(p1.xlen == 0 or len < p1.xlen) then
						-- remove last edge if this edge ylen not 0, edge already been added
						-- another point adding to same 2nd point
						if(p1.xlen ~= 0) then
							table.remove(edges,#edges)
						end
						p1.xlen = len
						p2.connectedx = true
						add_edge(p1.x,p1.y,p2.x,p2.y) -- edge already added reason for duplicates, adds everytime finds shorter
					end
				end

			end
		end

		--love.graphics.circle("fill",p1.x,p1.y,4)
	end

	-- remove edges off ground

	for i=#edges,1,-1 do
		local e = edges[i]
		if(e.y1 == e.y2) then
			if(on_ground2((e.x1+e.x2)/2,(e.y1+e.y2)/2) == false or on_ground2(e.x1+10, e.y1) == false or on_ground2(e.x2-10,e.y1) == false) then
				table.remove(edges,i)
			end
		end
	end

	-- remove edges fully on ground
	for i=#edges,1,-1 do
		local e = edges[i]
		if(e.x2 > e.x1 and on_ground2(e.x1 + 10, e.y1-10) == true and on_ground2(e.x1 + 10, e.y1+10) == true) then
			--love.graphics.rectangle("fill",e.x1 + 10, e.y1-10,1,1)
			--love.graphics.rectangle("fill",e.x1 + 10, e.y1+10,1,1) print
			table.remove(edges,i)
		end
	end	

	-- remove duplicate edges
	for i=#edges,1,-1 do
		local e1 = edges[i]	
		for j=#edges,1,-1 do
			local e2 = edges[j]	
			if not(i == j) then
				if(e1 and e1.x1 == e2.x1 and e1.x2 == e2.x2 and e1.y1 == e2.y1 and e1.y2 == e2.y2) then
					table.remove(edges,j)
				end
			end
		end
	end
end

function world_shadows(x, y, r)


	--world_createMesh()
	-- use horizontal line from y if player above, use horizontal line from y - wallHeight if player below

	local px, py = x, y
	py = py - 20
	

	--------------------------

	--[[
	-- doorway edges with wall height stuff, only draw over floor
	for i,v in ipairs(floors) do
		-- doorway edges
		add_edge(v.x,v.y+v.h-wallHeight,v.x,v.y+v.h)
		add_edge(v.x+v.w,v.y+v.h-wallHeight,v.x+v.w,v.y+v.h)
	end
	]]


	-- enemy shadows
	--[[
	for i=2,num_chars() do
		local char = get_char(i)
		if(char.active == true and on_screen(char.x - 50, char.y - 50, 100, 100)) then
			local d = dist(char.x, char.y, px, py)
			local dx, dy = math.cos(char.angle + math.pi) * 5000 / d, math.sin(char.angle + math.pi) * 5000 / d
			--add_edge(char.x - dx, char.y - dy, char.x + dx, char.x + dy)
			love.graphics.setLineWidth(20)
			love.graphics.line(char.x, char.y, char.x + dx, char.y + dy)
			love.graphics.setLineWidth(1)
			--love.graphics.circle("fill", char.x - dx, char.y - dy, 5)
			--love.graphics.circle("fill", char.x + dx, char.y + dy, 5)
		end
	end
	]]

	--- drawing
	--local px, py = get_playerPos()
	
	
	py = py - 20

	local shadowSize = 300

	for i=1,#edges do
		local e = edges[i]

		if(CheckCollision(e.x1,e.y1,e.x2-e.x1,e.y2-e.y1, px -r*50, py-r*50, r*100,r*100)) then

			--love.graphics.print(i, e.x1, e.y1)
			--love.graphics.line(e.x1,e.y1,e.x2,e.y2)

			if(px and py) then
				-- draw shadows
				local a1 = math.atan2(e.y1 - py, e.x1 - px)
				local dx1, dy1 = math.cos(a1) * shadowSize, math.sin(a1) * shadowSize
				local a2 = math.atan2(e.y2 - py, e.x2 - px)
				local dx2, dy2 = math.cos(a2) * shadowSize, math.sin(a2) * shadowSize

				--love.graphics.setColor(0,0,0)
				--love.graphics.line(e.x1,e.y1,e.x1+dx1,e.y1+dy1)
				--love.graphics.line(e.x2,e.y2,e.x2+dx2,e.y2+dy2)

				love.graphics.polygon("fill",e.x1,e.y1,e.x1+dx1,e.y1+dy1,e.x2+dx2,e.y2+dy2,e.x2,e.y2)

				-- extend tops of polygons away from light source to reach edge of screen
				if(e.y2 > e.y1) then
					local dx3, dy3 = 0, 0
					if(e.x1 < px) then
						dx3, dy3 = math.cos(math.pi) * shadowSize, math.sin(math.pi) * shadowSize
					else
						dx3, dy3 = math.cos(0) * shadowSize, math.sin(0) * shadowSize
					end
					love.graphics.polygon("fill",e.x1+dx1, e.y1+dy1, e.x1+dx1+dx3,e.y1+dy1+dy3,e.x2+dx2+dx3,e.y2+dy2+dy3,e.x2+dx2,e.y2+dy2)
				else
					if(e.x2 > e.x1) then
						local dx3, dy3 = 0, 0
						if(e.y1 > py) then
							dx3, dy3 = math.cos(math.pi/2) * shadowSize, math.sin(math.pi/2) * shadowSize
						else
							dx3, dy3 = math.cos(-math.pi/2) * shadowSize, math.sin(-math.pi/2) * shadowSize
						end
						love.graphics.polygon("fill",e.x1+dx1, e.y1+dy1, e.x1+dx1+dx3,e.y1+dy1+dy3,e.x2+dx2+dx3,e.y2+dy2+dy3,e.x2+dx2,e.y2+dy2)				
					end
				end


				--love.graphics.setColor(1,1,1)


			end
		end
	end
end

function soft_shadow()
	local px, py = get_playerPos()
	py = py - 40
	
	local r = 10
	local l = r * 100
	for i=1,#edges do
		local e = edges[i]
		local amount = 12
		for j=1,amount do
			love.graphics.setColor(0,0,0, (1 - j/amount) * 0.25)
			love.graphics.setLineWidth(j)
			if(e.x1 ~= e.x2) then
				love.graphics.line(e.x1 + j*j/2,e.y1,e.x2 - j*j/2,e.y2)
			else
				love.graphics.line(e.x1,e.y1 + j*j/2,e.x2,e.y2 - j*j/2)
			end
		end
		
		if(CheckCollision(e.x1,e.y1,e.x2-e.x1,e.y2-e.y1, px -r*50, py-r*50, r*100,r*100)) then
			love.graphics.setColor(1,1,1)
			--love.graphics.circle("line",e.x1,e.y1,5)
			
			
			local a = math.atan2(e.y1 - py,e.x1 - px)
			local dx, dy = math.cos(a), math.sin(a)		
			
			local amount = 10
			for j=1,amount do
				love.graphics.setColor(0,0,0, (1 - j/amount) * 0.25)
				love.graphics.setLineWidth(j)
				love.graphics.line(e.x1 + dx * j*j/2, e.y1 + dy * j*j/2, e.x1 + dx * l, e.y1 + dy * l)
				--love.graphics.circle("fill", e.x1, e.y1, j/4)
			end
			
			
			local a = math.atan2(e.y2 - py,e.x2 - px)
			local dx, dy = math.cos(a), math.sin(a)
			
			
			local amount = 10
			for j=1,amount do
				love.graphics.setColor(0,0,0, (1 - j/amount) * 0.25)
				love.graphics.setLineWidth(j)
				--love.graphics.line(e.x2, e.y2, e.x2 + dx, e.y2 + dy)
				love.graphics.line(e.x2 + dx * j*j/2, e.y2 + dy * j*j/2, e.x2 + dx * l, e.y2 + dy * l)
				--love.graphics.circle("fill", e.x2, e.y2, j/4)
				--love.graphics.circle("fill", e.x1, e.y1, j)
			end			
		end
	end
	
	love.graphics.setColor(1,1,1)
	love.graphics.setLineWidth(1)	
end

local fountainImg = love.graphics.newImage("models/fountain.png")
local gateTile = love.graphics.newImage("gateTile.png")
local switchImg = love.graphics.newImage("switch1.png")
local switchImg2 = love.graphics.newImage("switch2.png")
function world_draw()
	love.graphics.print(#edges,500,500)
	for i,v in ipairs(floors) do -- wall top
		love.graphics.setColor(0.25,0.25,0.25)
		love.graphics.rectangle("fill", v.x - wallTopWidth ,v.y - wallTopWidth - wallHeight, v.w + wallTopWidth*2,v.h + wallTopWidth *2 + wallHeight)

		love.graphics.setColor(0,0,0)
		love.graphics.setLineWidth(4)
		--love.graphics.rectangle("line", v.x - wallTopWidth/2, v.y - wallTopWidth/2 - wallHeight, v.w + wallTopWidth, v.h + wallHeight + wallTopWidth)
		love.graphics.setLineWidth(1)
		love.graphics.setColor(1,1,1)

	end

	for i,v in ipairs(floors) do -- wall
		
		if(on_screen(v.x,v.y - wallHeight,v.w,wallHeight)) then
			love.graphics.setColor(0.5,0.5,0.5)
			for x=0,v.w / wallTile:getWidth() - 1 do
				if(v.hasFountain) then--if(i % 2 == 0) then
					love.graphics.draw(wallTile2, v.x + x * wallTile:getWidth(), v.y - wallHeight)
				else
					love.graphics.draw(wallTile, v.x + x * wallTile:getWidth(), v.y - wallHeight)
				end
			end
			love.graphics.setColor(0,0,0,1 - v.wallReveal)
			--love.graphics.rectangle("fill",v.x,v.y-wallHeight,v.w,wallHeight) ---------------------------------------------- < 
		end
		--love.graphics.rectangle("fill",v.x,v.y-wallHeight,v.w,wallHeight)
		--add_drawable("rect", {x=v.x,y=v.y - wallHeight,w=v.w,h=wallHeight, height = 100})
	end

	love.graphics.setColor(1,1,1)
	for i,v in ipairs(floors) do -- floor
		if(state == "edit" or on_screen(v.x,v.y,v.w,v.h)) then
			
			for y=0,v.h / floorTile:getHeight() - 1 do
				for x=0,v.w / floorTile:getWidth() - 1 do
					if(state == "edit" or on_screen(v.x + x * floorTile:getWidth(), v.y + y * floorTile:getHeight(), floorTile:getWidth(), floorTile:getHeight())) then
						if(v.hasFountain) then--if(i % 2 == 0) then
							love.graphics.draw(floorTile2, v.x + x * floorTile:getWidth(), v.y + y * floorTile:getHeight())
						else
							love.graphics.draw(floorTile, v.x + x * floorTile:getWidth(), v.y + y * floorTile:getHeight())
						end
					end
				end
			end

			love.graphics.setBlendMode("alpha","premultiplied")
			love.graphics.draw(v.canvas,v.x,v.y)
			love.graphics.setBlendMode("alpha")



			love.graphics.setColor(0,0,0,floorFadeOut)
			love.graphics.rectangle("fill",v.x,v.y,v.w,v.h)
			love.graphics.setColor(1,1,1)

			--love.graphics.rectangle("fill",v.x,v.y,v.w,v.h)
		end
	end

	for i,v in ipairs(floors) do
		if(on_screen(v.x,v.y,v.w,v.h)) then
			if(v.hasFountain) then
				love.graphics.setLineWidth(4)
				love.graphics.setColor(0.4,0.4,0.4)
				love.graphics.rectangle("line",v.x-2,v.y-2,v.w+4,v.h+4)
				love.graphics.setLineWidth(2)
				love.graphics.setColor(0.5,0.5,0.5,1)
				love.graphics.rectangle("line",v.x,v.y,v.w,v.h)
				love.graphics.setColor(1,1,1)
			end
		end
	end

	
	for i,v in ipairs(gates) do
		if(on_screen(v.x,v.y,v.w,v.h)) then
		
			--love.graphics.rectangle("line", v.x, v.y, v.w, v.h)
			local tw = gateTile:getWidth()
			local th = gateTile:getHeight()
			for x = 0, v.w/tw - 1 do
				for y = 0, v.h/tw - 1 do
					love.graphics.draw(gateTile, v.x + x * tw, v.y + y * tw)
					if(v.closed) then
						model_draw("spike", v.x + x * tw + tw/2, v.y + y * tw + tw/2, 0, 1, 1, 0.5, 0.5, 0)
					end
					--add_drawable("img", {img = gateImg, x=v.x + x * tw,y=v.y + y * tw - th + tw,angle=0,sx=1,sy=1,ox=0,oy=0,height= y})
				end
			end
			
			if(state == "edit") then
				love.graphics.print(v.id, v.x, v.y - 10)
			end			
		end
	end
	
	
	for i,v in ipairs(switches) do
		if (on_screen(v.x - 32, v.y - 32, 64, 64)) then
			if(v.pressed == false) then
				love.graphics.draw(switchImg, v.x - 16, v.y - 32 - 16)
			else
				love.graphics.draw(switchImg2, v.x - 16, v.y - 32 - 16)
			end
			--love.graphics.circle("fill", v.x, v.y, 10)
			if(state == "edit") then
				love.graphics.print(v.id, v.x, v.y - 10)
			end
		end
	end

end

function world_wallTopStencil(offset)
	for i,v in ipairs(floors) do
		love.graphics.rectangle("fill", v.x - offset, v.y - offset - wallHeight, v.w + offset*2, v.h + offset*2 + wallHeight)
	end
end



function world_wallTops()
	for i,v in ipairs(floors) do -- wall top
		for j = 1,wallTopWidth do
			love.graphics.setColor(0,0,0,j/wallTopWidth)
			love.graphics.rectangle("line", v.x - j ,v.y - j - wallHeight, v.w + j*2,v.h + j *2 + wallHeight)
		end
		
		love.graphics.setColor(0,0,0)
		love.graphics.setLineWidth(4)
		--love.graphics.rectangle("line", v.x - wallTopWidth/2, v.y - wallTopWidth/2 - wallHeight, v.w + wallTopWidth, v.h + wallHeight + wallTopWidth)
		love.graphics.setLineWidth(1)
		love.graphics.setColor(1,1,1)

	end
end

function world_drawFountains()
	for i,v in ipairs(fountains) do
		love.graphics.setColor(0,0,0,0.1)
		for i=1,5 do
			love.graphics.circle("fill",v.x, v.y + 20, 120 - i)
		end
		love.graphics.setColor(1,1,1)
		--model_draw("fountain", start.x, start.y ,0, 1, 1, 0.5, 0.5, 0)
		love.graphics.draw(fountainImg,v.x - 100,v.y - 200,0,1,1,0,0)
	end
end

function world_reset()
	--light_floors()
	
	floorFadeOut = 0
	--floors = {}
end

function fountain_setup()
	--fountains = {}
	--add_fountain(start.x, start.y)
	--add_fountain(exit.x, exit.y)

	for i,v in ipairs(floors) do
		for j,k in ipairs(fountains) do
			if(CheckCollision(v.x,v.y,v.w,v.h,k.x,k.y,1,1)) then
				v.hasFountain = true
			end
		end
	end
end